unit frmMain;
// This file is part of Sarah Dean's Code Co-op Diff Launcher.
//
//  "Sarah Dean's Code Co-op Diff Launcher" is free software; you can
// redistribute it and/or modify it under the terms of the GNU General
// Public License as published by the Free Software Foundation; either
// version 2 of the License, or at your option) any later version.
//
//  "Sarah Dean's Code Co-op Diff Launcher" is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with "Sarah Dean's Code Co-op Diff Launcher"; if not, write to
// the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
// MA  02111-1307  USA


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,
  SDCCDLIniFile_U, ComCtrls;

type
  TMain = class(TForm)
    edDiffCommand: TEdit;
    Label1: TLabel;
    pbSet: TButton;
    pbBrowse: TButton;
    ckDebug: TCheckBox;
    OpenDialog1: TOpenDialog;
    reInstructions: TRichEdit;
    pbClose: TButton;
    edDiffParameters: TEdit;
    Label2: TLabel;
    pbAbout: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure pbSetClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure pbBrowseClick(Sender: TObject);
    procedure pbCloseClick(Sender: TObject);
    procedure pbAboutClick(Sender: TObject);
  private
    Config: TSDCCDLIniFile;
    procedure SetAltDiffToThis();

  public
    { Public declarations }
  end;

var
  Main: TMain;

implementation

{$R *.DFM}

uses
  Registry,
  SDUGeneral,
  About_U,
  AppGlobalConsts_U;


procedure TMain.FormCreate(Sender: TObject);
var
  diffCmd: string;
  x: integer;
  i: integer;
  pcntParam: string;
  debugMsg: string;
  StartupInfo:TStartupInfo;
  ProcessInfo:TProcessInformation;
begin
  Config:= TSDCCDLIniFile.Create(INI_FILENAME);
  Config.ReadIniFile();

  self.Caption := APP_TITLE;

  if (ParamCount>0) then
    begin
    if Config.OptDebug then
      begin
      showmessage('Command line parameters received on startup:'+CRLF+CRLF+CmdLine);

      debugMsg := 'Parameter mapping:'+CRLF;
      for i:=1 to ParamCount do
        begin
        debugMsg := debugMsg+CRLF+
                    '%'+inttostr(i)+' -> '+ParamStr(i);
        end;

      showmessage(debugMsg);
      end;

    diffCmd := '"'+Config.OptDiffCommand+'" ' + Config.OptDiffParameters;

    for i:=1 to ParamCount do
      begin
      pcntParam := '%'+inttostr(i);
      x := Pos(pcntParam, diffCmd);
      while (x > 0) do
        begin
        Delete(diffCmd, x, length(pcntParam));
        Insert('"'+ParamStr(i)+'"', diffCmd, x);

        x := Pos(pcntParam, diffCmd);
        end;

      end;


    FillChar(StartupInfo, Sizeof(StartupInfo), #0);
    StartupInfo.cb := Sizeof(StartupInfo);

    StartupInfo.dwFlags := STARTF_USESHOWWINDOW;
    StartupInfo.wShowWindow := SW_SHOW;// cmdShow;


    if Config.OptDebug then
      begin
      showmessage('Diff command to be used is:'+CRLF+CRLF+diffCmd);
      end
    else
      begin
      if CreateProcess(nil,
                       PChar(diffCmd),       { pointer to command line string }
                       nil,                   { pointer to process security attributes }
                       nil,                   { pointer to thread security attributes }
                       FALSE,                 { handle inheritance flag }
                       CREATE_NEW_CONSOLE or NORMAL_PRIORITY_CLASS,
                       nil,                   { pointer to new environment block }
                       nil,                   { pointer to current directory name }
                       StartupInfo,           { pointer to STARTUPINFO }
                       ProcessInfo) then     { pointer to PROCESS_INF }
        begin
        Close;
        Application.Terminate;
        end
      else
        begin
        showmessage('Failed to launch diff program with command:'+CRLF+CRLF+diffCmd);
        end;

      end;

    end;


end;


procedure TMain.FormDestroy(Sender: TObject);
begin
  Config.Free();

end;

procedure TMain.pbSetClick(Sender: TObject);
begin
  Config.OptDebug := ckDebug.checked;
  Config.OptDiffCommand := edDiffCommand.Text;
  Config.OptDiffParameters := edDiffParameters.Text;
  Config.WriteIniFile();

  SetAltDiffToThis();

  MessageDlg('Diff Launcher configured.', mtInformation, [mbOK], 0);

end;

procedure TMain.FormShow(Sender: TObject);
begin
  ckDebug.checked := Config.OptDebug;
  edDiffCommand.Text := Config.OptDiffCommand;
  edDiffParameters.Text := Config.OptDiffParameters;

end;

procedure TMain.pbBrowseClick(Sender: TObject);
begin
  SDUOpenSaveDialogSetup(OpenDialog1, edDiffCommand.text);
  
  if OpenDialog1.Execute() then
    begin
    edDiffCommand.text := OpenDialog1.Filename;
    end;

end;

procedure TMain.pbCloseClick(Sender: TObject);
begin
  Close();
  
end;

procedure TMain.pbAboutClick(Sender: TObject);
var
  aboutDlg: TAbout_F;
begin
  aboutDlg:= TAbout_F.Create(nil);
  try
    aboutDlg.ShowModal();
  finally
    aboutDlg.Free();
  end;

end;

procedure TMain.SetAltDiffToThis();
const
  REGKEY_CODECOOP_ALT_DIFF = '\Software\Reliable Software\Code Co-op\Preferences\Differ\Alternative';
var
  registry: TRegistry;
begin
  try
    registry:= TRegistry.Create();
    try
      registry.RootKey := HKEY_CURRENT_USER;
      registry.OpenKey(REGKEY_CODECOOP_ALT_DIFF, True);
      registry.WriteString('Path', ParamStr(0));
      registry.WriteBool('On', TRUE);
    finally
      registry.Free;
    end;


  except
    on ERegistryException do
      begin
      MessageDlg('An error occured while writing to the registry;'+#13+#10+'please ensure that you have the necessary permissions do perform '+#13+#10+'this operation (i.e. logon as administrator) and try again', mtError, [mbOK], 0);
      end;
  end;

end;


END.


